@@ -176,7 +176,7 @@ exit_cleanup:
 	return res;
 }
 
-int init_data(struct data *data)
+static int init_data(struct data *data)
 {
 	int res;
 	const char *str;
@@ -832,7 +832,7 @@ static int negotiate_formats(struct data *data)
 	if ((res = spa_node_port_use_buffers(data->source_node,
 		SPA_DIRECTION_OUTPUT, 0, 0, data->source_buffers, 1)) < 0)
 		return res;
-	printf("allocated and assigned buffer(%ld) to source node %p\n", buffer_size, data->source_node);
+	printf("allocated and assigned buffer (%zu) to source node %p\n", buffer_size, data->source_node);
 	if ((res = spa_node_port_use_buffers(data->sink_node,
 		SPA_DIRECTION_INPUT, 0, 0, data->source_buffers, 1)) < 0)
 		return res;
@@ -906,9 +906,9 @@ static void run_async_sink(struct data *data)
 		printf("got error %d\n", res);
 }
 
-static char *getscale(uint32_t scale)
+static const char *getscale(uint32_t scale)
 {
-	char *scale_s = NULL;
+	const char *scale_s = NULL;
 
 	if (scale == SPA_AUDIO_VOLUME_RAMP_LINEAR)
 		scale_s = LINEAR;
