@@ -31,9 +31,10 @@
 
 #define MAX_CARDS	64
 
-#define ACTION_ADD	0
-#define ACTION_REMOVE	1
-#define ACTION_DISABLE	2
+enum action {
+	ACTION_CHANGE,
+	ACTION_REMOVE,
+};
 
 /* Used for unavailable devices in the card structure. */
 #define ID_DEVICE_NOT_SUPPORTED 0
@@ -93,6 +94,7 @@ struct impl {
 	struct spa_source notify;
 	unsigned int use_acp:1;
 	unsigned int expose_busy:1;
+	int use_ucm;
 };
 
 static int impl_udev_open(struct impl *this)
@@ -499,7 +501,7 @@ static int emit_added_object_info(struct impl *this, struct card *card)
 	if (num_pcm_devices > 0) {
 		struct spa_device_object_info info;
 		char *cn = NULL, *cln = NULL;
-		struct spa_dict_item items[25];
+		struct spa_dict_item items[26];
 		unsigned int n_items = 0;
 
 		card->pcm_device_id = calc_pcm_device_id(card);
@@ -520,6 +522,9 @@ static int emit_added_object_info(struct impl *this, struct card *card)
 		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_DEVICE_ENUM_API, "udev");
 		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_DEVICE_API,  "alsa");
 		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_MEDIA_CLASS, "Audio/Device");
+		if (this->use_ucm != -1)
+			items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_API_ALSA_USE_UCM,
+					this->use_ucm ? "true" : "false");
 		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_API_ALSA_PATH, path);
 		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_API_ALSA_CARD, path+3);
 		if (snd_card_get_name(card->card_nr, &cn) >= 0)
@@ -718,57 +723,48 @@ static bool check_access(struct impl *this, struct card *card)
 	return card->accessible;
 }
 
-static void process_card(struct impl *this, uint32_t action, struct udev_device *udev_device)
+static void process_card(struct impl *this, enum action action, struct card *card)
 {
-	unsigned int card_nr;
-	struct card *card;
-	bool emitted;
-	int res;
-
-	if ((card_nr = get_card_nr(this, udev_device)) == SPA_ID_INVALID)
-		return;
-
-	card = find_card(this, card_nr);
-	if (card && card->ignored)
+	if (card->ignored)
 		return;
 
 	switch (action) {
-	case ACTION_ADD:
-		if (card == NULL)
-			card = add_card(this, card_nr, udev_device);
-		if (card == NULL)
-			return;
-		if (!check_access(this, card))
-			return;
-		res = emit_added_object_info(this, card);
-		if (res < 0) {
-			if (card->ignored)
-				spa_log_info(this->log, "ALSA card %u unavailable (%s): it is ignored",
-						card->card_nr, spa_strerror(res));
-			else if (!card->unavailable)
-				spa_log_info(this->log, "ALSA card %u unavailable (%s): wait for it",
-						card->card_nr, spa_strerror(res));
-			else
-				spa_log_debug(this->log, "ALSA card %u still unavailable (%s)",
-						card->card_nr, spa_strerror(res));
-			card->unavailable = true;
-		} else {
-			if (card->unavailable)
-				spa_log_info(this->log, "ALSA card %u now available",
-						card->card_nr);
-			card->unavailable = false;
+	case ACTION_CHANGE: {
+		check_access(this, card);
+		if (card->accessible && !card->emitted) {
+			int res = emit_added_object_info(this, card);
+			if (res < 0) {
+				if (card->ignored)
+					spa_log_info(this->log, "ALSA card %u unavailable (%s): it is ignored",
+							card->card_nr, spa_strerror(res));
+				else if (!card->unavailable)
+					spa_log_info(this->log, "ALSA card %u unavailable (%s): wait for it",
+							card->card_nr, spa_strerror(res));
+				else
+					spa_log_debug(this->log, "ALSA card %u still unavailable (%s)",
+							card->card_nr, spa_strerror(res));
+				card->unavailable = true;
+			} else {
+				if (card->unavailable)
+					spa_log_info(this->log, "ALSA card %u now available",
+							card->card_nr);
+				card->unavailable = false;
+			}
+		} else if (!card->accessible && card->emitted) {
+			card->emitted = false;
+
+			if (card->pcm_device_id != ID_DEVICE_NOT_SUPPORTED)
+				spa_device_emit_object_info(&this->hooks, card->pcm_device_id, NULL);
+			if (card->compress_offload_device_id != ID_DEVICE_NOT_SUPPORTED)
+				spa_device_emit_object_info(&this->hooks, card->compress_offload_device_id, NULL);
 		}
 		break;
-
+	}
 	case ACTION_REMOVE: {
-		uint32_t pcm_device_id, compress_offload_device_id;
+		uint32_t pcm_device_id = card->pcm_device_id;
+		uint32_t compress_offload_device_id = card->compress_offload_device_id;
+		bool emitted = card->emitted;
 
-		if (card == NULL)
-			return;
-
-		emitted = card->emitted;
-		pcm_device_id = card->pcm_device_id;
-		compress_offload_device_id = card->compress_offload_device_id;
 		remove_card(this, card);
 
 		if (emitted) {
@@ -779,27 +775,27 @@ static void process_card(struct impl *this, uint32_t action, struct udev_device
 		}
 		break;
 	}
-
-	case ACTION_DISABLE:
-		if (card == NULL)
-			return;
-		if (card->emitted) {
-			uint32_t pcm_device_id, compress_offload_device_id;
-
-			pcm_device_id = card->pcm_device_id;
-			compress_offload_device_id = card->compress_offload_device_id;
-
-			card->emitted = false;
-
-			if (pcm_device_id != ID_DEVICE_NOT_SUPPORTED)
-				spa_device_emit_object_info(&this->hooks, pcm_device_id, NULL);
-			if (compress_offload_device_id != ID_DEVICE_NOT_SUPPORTED)
-				spa_device_emit_object_info(&this->hooks, compress_offload_device_id, NULL);
-		}
-		break;
 	}
 }
 
+static void process_udev_device(struct impl *this, enum action action, struct udev_device *udev_device)
+{
+	unsigned int card_nr;
+	struct card *card;
+
+	if ((card_nr = get_card_nr(this, udev_device)) == SPA_ID_INVALID)
+		return;
+
+	card = find_card(this, card_nr);
+	if (action == ACTION_CHANGE && !card)
+		card = add_card(this, card_nr, udev_device);
+
+	if (!card)
+		return;
+
+	process_card(this, action, card);
+}
+
 static int stop_inotify(struct impl *this)
 {
 	if (this->notify.fd == -1)
@@ -826,8 +822,6 @@ static void impl_on_notify_events(struct spa_source *source)
 		void *p, *e;
 
 		len = read(source->fd, &buf, sizeof(buf));
-		if (len < 0 && errno != EAGAIN)
-			break;
 		if (len <= 0)
 			break;
 
@@ -845,21 +839,16 @@ static void impl_on_notify_events(struct spa_source *source)
 
 			/* card becomes accessible or not busy */
 			if ((event->mask & (IN_ATTRIB | IN_CLOSE_WRITE))) {
-				bool access;
 				if (sscanf(event->name, "controlC%u", &card_nr) != 1 &&
 				    sscanf(event->name, "pcmC%uD", &card_nr) != 1)
 					continue;
 				if ((card = find_card(this, card_nr)) == NULL)
 					continue;
 
-				access = check_access(this, card);
-				if (access && !card->emitted)
-					process_card(this, ACTION_ADD, card->udev_device);
-				else if (!access && card->emitted)
-					process_card(this, ACTION_DISABLE, card->udev_device);
+				process_card(this, ACTION_CHANGE, card);
 			}
 			/* /dev/snd/ might have been removed */
-			if ((event->mask & (IN_DELETE_SELF | IN_MOVE_SELF)))
+			if ((event->mask & (IN_IGNORED | IN_MOVE_SELF)))
 				deleted = true;
 		}
 	}
@@ -878,7 +867,7 @@ static int start_inotify(struct impl *this)
 		return -errno;
 
 	res = inotify_add_watch(notify_fd, "/dev/snd",
-			IN_ATTRIB | IN_CLOSE_WRITE | IN_DELETE_SELF | IN_MOVE_SELF);
+			IN_ATTRIB | IN_CLOSE_WRITE | IN_MOVE_SELF);
 	if (res < 0) {
 		res = -errno;
 		close(notify_fd);
@@ -918,10 +907,10 @@ static void impl_on_fd_events(struct spa_source *source)
 
 	start_inotify(this);
 
-	if (spa_streq(action, "change")) {
-		process_card(this, ACTION_ADD, udev_device);
+	if (spa_streq(action, "add") || spa_streq(action, "change")) {
+		process_udev_device(this, ACTION_CHANGE, udev_device);
 	} else if (spa_streq(action, "remove")) {
-		process_card(this, ACTION_REMOVE, udev_device);
+		process_udev_device(this, ACTION_REMOVE, udev_device);
 	}
 	udev_device_unref(udev_device);
 }
@@ -992,7 +981,7 @@ static int enum_cards(struct impl *this)
 		if (udev_device == NULL)
 			continue;
 
-		process_card(this, ACTION_ADD, udev_device);
+		process_udev_device(this, ACTION_CHANGE, udev_device);
 
 		udev_device_unref(udev_device);
 	}
@@ -1120,6 +1109,7 @@ impl_init(const struct spa_handle_factory *factory,
 	alsa_log_topic_init(this->log);
 	this->main_loop = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Loop);
 	this->main_system = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_System);
+	this->use_ucm = -1;
 
 	if (this->main_loop == NULL) {
 		spa_log_error(this->log, "a main-loop is needed");
@@ -1146,6 +1136,8 @@ impl_init(const struct spa_handle_factory *factory,
 			this->use_acp = spa_atob(str);
 		else if ((str = spa_dict_lookup(info, "alsa.udev.expose-busy")) != NULL)
 			this->expose_busy = spa_atob(str);
+		else if ((str = spa_dict_lookup(info, "alsa.use-ucm")) != NULL)
+			this->use_ucm = spa_atob(str) ? 1 : 0;
 	}
 
 	return 0;
