@@ -23,7 +23,9 @@
 #include <spa/param/param.h>
 #include <spa/pod/filter.h>
 
-#define NAME "videotestsrc"
+#undef SPA_LOG_TOPIC_DEFAULT
+#define SPA_LOG_TOPIC_DEFAULT &log_topic
+SPA_LOG_TOPIC_DEFINE_STATIC(log_topic, "spa.videotestsrc");
 
 #define FRAMES_TO_TIME(port,f) ((port->current_format.info.raw.framerate.denom * (f) * SPA_NSEC_PER_SEC) / \
                                 (port->current_format.info.raw.framerate.num))
@@ -82,7 +84,7 @@ struct impl {
 
 	struct spa_log *log;
 	struct spa_loop *data_loop;
-	struct spa_system *data_system;
+	struct spa_loop_utils *loop_utils;
 
 	uint64_t info_all;
 	struct spa_node_info info;
@@ -96,7 +98,7 @@ struct impl {
 	struct spa_callbacks callbacks;
 
 	bool async;
-	struct spa_source timer_source;
+	struct spa_source *timer_source;
 	struct itimerspec timerspec;
 
 	bool started;
@@ -275,27 +277,10 @@ static void set_timer(struct impl *this, bool enabled)
 			this->timerspec.it_value.tv_sec = 0;
 			this->timerspec.it_value.tv_nsec = 0;
 		}
-		spa_system_timerfd_settime(this->data_system,
-				this->timer_source.fd, SPA_FD_TIMER_ABSTIME, &this->timerspec, NULL);
+		spa_loop_utils_update_timer(this->loop_utils, this->timer_source, &this->timerspec.it_value, &this->timerspec.it_interval, true);
 	}
 }
 
-static int read_timer(struct impl *this)
-{
-	uint64_t expirations;
-	int res = 0;
-
-	if (this->async || this->props.live) {
-		if ((res = spa_system_timerfd_read(this->data_system,
-						this->timer_source.fd, &expirations)) < 0) {
-			if (res != -EAGAIN)
-				spa_log_error(this->log, NAME " %p: timerfd error: %s",
-						this, spa_strerror(res));
-		}
-	}
-	return res;
-}
-
 static int make_buffer(struct impl *this)
 {
 	struct buffer *b;
@@ -303,12 +288,9 @@ static int make_buffer(struct impl *this)
 	struct spa_io_buffers *io = port->io;
 	uint32_t n_bytes;
 
-	if (read_timer(this) < 0)
-		return 0;
-
 	if (spa_list_is_empty(&port->empty)) {
 		set_timer(this, false);
-		spa_log_error(this->log, NAME " %p: out of buffers", this);
+		spa_log_error(this->log, "%p: out of buffers", this);
 		return -EPIPE;
 	}
 	b = spa_list_first(&port->empty, struct buffer, link);
@@ -317,7 +299,7 @@ static int make_buffer(struct impl *this)
 
 	n_bytes = b->outbuf->datas[0].maxsize;
 
-	spa_log_trace(this->log, NAME " %p: dequeue buffer %d", this, b->id);
+	spa_log_trace(this->log, "%p: dequeue buffer %d", this, b->id);
 
 	fill_buffer(this, b);
 
@@ -341,9 +323,9 @@ static int make_buffer(struct impl *this)
 	return io->status;
 }
 
-static void on_output(struct spa_source *source)
+static void on_output(void* data, uint64_t expirations)
 {
-	struct impl *this = source->data;
+	struct impl *this = data;
 	int res;
 
 	res = make_buffer(this);
@@ -612,7 +594,7 @@ impl_node_port_enum_params(void *object, int seq,
 static int clear_buffers(struct impl *this, struct port *port)
 {
 	if (port->n_buffers > 0) {
-		spa_log_debug(this->log, NAME " %p: clear buffers", this);
+		spa_log_debug(this->log, "%p: clear buffers", this);
 		port->n_buffers = 0;
 		spa_list_init(&port->empty);
 		this->started = false;
@@ -729,7 +711,7 @@ impl_node_port_use_buffers(void *object,
 		b->h = spa_buffer_find_meta_data(buffers[i], SPA_META_Header, sizeof(*b->h));
 
 		if (d[0].data == NULL) {
-			spa_log_error(this->log, NAME " %p: invalid memory on buffer %p", this,
+			spa_log_error(this->log, "%p: invalid memory on buffer %p", this,
 				      buffers[i]);
 			return -EINVAL;
 		}
@@ -769,7 +751,7 @@ static inline void reuse_buffer(struct impl *this, struct port *port, uint32_t i
 	struct buffer *b = &port->buffers[id];
 	spa_return_if_fail(b->outstanding);
 
-	spa_log_trace(this->log, NAME " %p: reuse buffer %d", this, id);
+	spa_log_trace(this->log, "%p: reuse buffer %d", this, id);
 
 	b->outstanding = false;
 	spa_list_append(&port->empty, &b->link);
@@ -857,7 +839,7 @@ static int impl_get_interface(struct spa_handle *handle, const char *type, void
 static int do_remove_timer(struct spa_loop *loop, bool async, uint32_t seq, const void *data, size_t size, void *user_data)
 {
 	struct impl *this = user_data;
-	spa_loop_remove_source(this->data_loop, &this->timer_source);
+	spa_loop_remove_source(this->data_loop, this->timer_source);
 	return 0;
 }
 
@@ -871,7 +853,7 @@ static int impl_clear(struct spa_handle *handle)
 
 	if (this->data_loop)
 		spa_loop_invoke(this->data_loop, do_remove_timer, 0, NULL, 0, true, this);
-	spa_system_close(this->data_system, this->timer_source.fd);
+	spa_loop_utils_destroy_source(this->loop_utils, this->timer_source);
 
 	return 0;
 }
@@ -903,7 +885,7 @@ impl_init(const struct spa_handle_factory *factory,
 
 	this->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
 	this->data_loop = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_DataLoop);
-	this->data_system = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_DataSystem);
+	this->loop_utils = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_LoopUtils);
 
 	spa_hook_list_init(&this->hooks);
 
@@ -924,20 +906,12 @@ impl_init(const struct spa_handle_factory *factory,
 	this->info.n_params = 2;
 	reset_props(&this->props);
 
-	this->timer_source.func = on_output;
-	this->timer_source.data = this;
-	this->timer_source.fd = spa_system_timerfd_create(this->data_system, CLOCK_MONOTONIC,
-							  SPA_FD_CLOEXEC | SPA_FD_NONBLOCK);
-	this->timer_source.mask = SPA_IO_IN;
-	this->timer_source.rmask = 0;
+	this->timer_source = spa_loop_utils_add_timer(this->loop_utils, on_output, this);
 	this->timerspec.it_value.tv_sec = 0;
 	this->timerspec.it_value.tv_nsec = 0;
 	this->timerspec.it_interval.tv_sec = 0;
 	this->timerspec.it_interval.tv_nsec = 0;
 
-	if (this->data_loop)
-		spa_loop_add_source(this->data_loop, &this->timer_source);
-
 	port = &this->port;
 	port->info_all = SPA_PORT_CHANGE_MASK_FLAGS |
 			SPA_PORT_CHANGE_MASK_PARAMS;
@@ -990,7 +964,7 @@ static const struct spa_dict info = SPA_DICT_INIT_ARRAY(info_items);
 
 const struct spa_handle_factory spa_videotestsrc_factory = {
 	SPA_VERSION_HANDLE_FACTORY,
-	NAME,
+	"videotestsrc",
 	&info,
 	impl_get_size,
 	impl_init,
