@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2018 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2018 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 /*
  [title]
@@ -34,8 +14,11 @@
 
 #include <spa/utils/result.h>
 #include <spa/param/video/format-utils.h>
+#include <spa/param/tag-utils.h>
 #include <spa/param/props.h>
+#include <spa/param/latency-utils.h>
 #include <spa/debug/format.h>
+#include <spa/debug/pod.h>
 
 #include <pipewire/pipewire.h>
 
@@ -107,6 +90,7 @@ on_process(void *_data)
 	int sstride, dstride, ostride;
 	struct spa_meta_region *mc;
 	struct spa_meta_cursor *mcs;
+	struct spa_meta_header *h;
 	uint32_t i, j;
 	uint8_t *src, *dst;
 	bool render_cursor = false;
@@ -134,6 +118,12 @@ on_process(void *_data)
 	if ((sdata = buf->datas[0].data) == NULL)
 		goto done;
 
+	if ((h = spa_buffer_find_meta_data(buf, SPA_META_Header, sizeof(*h)))) {
+		uint64_t now = pw_stream_get_nsec(stream);
+		pw_log_debug("now:%"PRIu64" pts:%"PRIu64" diff:%"PRIi64,
+				now, h->pts, now - h->pts);
+	}
+
 	/* get the videocrop metadata if any */
 	if ((mc = spa_buffer_find_meta_data(buf, SPA_META_VideoCrop, sizeof(*mc))) &&
 	    spa_meta_region_is_valid(mc)) {
@@ -215,10 +205,10 @@ on_process(void *_data)
 			for (i = 0; i < data->size.height; i++) {
 				struct pixel *p = (struct pixel *) src;
 				for (j = 0; j < data->size.width; j++) {
-					dst[j * 4 + 0] = SPA_CLAMP(p[j].r * 255.0f, 0, 255);
-					dst[j * 4 + 1] = SPA_CLAMP(p[j].g * 255.0f, 0, 255);
-					dst[j * 4 + 2] = SPA_CLAMP(p[j].b * 255.0f, 0, 255);
-					dst[j * 4 + 3] = SPA_CLAMP(p[j].a * 255.0f, 0, 255);
+					dst[j * 4 + 0] = SPA_CLAMP((uint8_t)(p[j].r * 255.0f), 0u, 255u);
+					dst[j * 4 + 1] = SPA_CLAMP((uint8_t)(p[j].g * 255.0f), 0u, 255u);
+					dst[j * 4 + 2] = SPA_CLAMP((uint8_t)(p[j].b * 255.0f), 0u, 255u);
+					dst[j * 4 + 3] = SPA_CLAMP((uint8_t)(p[j].a * 255.0f), 0u, 255u);
 				}
 				src += sstride;
 				dst += dstride;
@@ -298,6 +288,16 @@ on_stream_param_changed(void *_data, uint32_t id, const struct spa_pod *param)
 	void *d;
 	int32_t mult, size;
 
+	if (param != NULL && id == SPA_PARAM_Tag) {
+		spa_debug_pod(0, NULL, param);
+		return;
+	}
+	if (param != NULL && id == SPA_PARAM_Latency) {
+		struct spa_latency_info info;
+		if (spa_latency_parse(param, &info) >= 0)
+			fprintf(stderr, "got latency: %"PRIu64"\n", (info.min_ns + info.max_ns) / 2);
+		return;
+	}
 	/* NULL means to clear the format */
 	if (param == NULL || id != SPA_PARAM_Format)
 		return;
@@ -442,7 +442,7 @@ static void do_quit(void *userdata, int signal_number)
 int main(int argc, char *argv[])
 {
 	struct data data = { 0, };
-	const struct spa_pod *params[2];
+	const struct spa_pod *params[3];
 	uint8_t buffer[1024];
 	struct spa_pod_builder b = SPA_POD_BUILDER_INIT(buffer, sizeof(buffer));
 	struct pw_properties *props;
@@ -498,6 +498,16 @@ int main(int argc, char *argv[])
 	 * object to the stack. */
 	n_params = build_format(&data, &b, params);
 
+	{
+		struct spa_pod_frame f;
+		struct spa_dict_item items[1];
+		/* send a tag, input tags travel upstream */
+		spa_tag_build_start(&b, &f, SPA_PARAM_Tag, SPA_DIRECTION_INPUT);
+		items[0] = SPA_DICT_ITEM_INIT("my-tag-other-key", "my-special-other-tag-value");
+		spa_tag_build_add_dict(&b, &SPA_DICT_INIT(items, 1));
+		params[n_params++] = spa_tag_build_end(&b, &f);
+	}
+
 	/* now connect the stream, we need a direction (input/output),
 	 * an optional target node to connect to, some flags and parameters
 	 */
