@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2018 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2018 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include <errno.h>
 
@@ -432,6 +412,9 @@ static int client_node_demarshal_event_event(void *data, const struct pw_protoco
 				SPA_POD_PodObject(&event)) < 0)
 		return -EINVAL;
 
+	if (event == NULL)
+		return -EINVAL;
+
 	pw_proxy_notify(proxy, struct pw_client_node_events, event, 0, event);
 	return 0;
 }
@@ -447,6 +430,9 @@ static int client_node_demarshal_command(void *data, const struct pw_protocol_na
 			SPA_POD_PodObject(&command)) < 0)
 		return -EINVAL;
 
+	if (command == NULL)
+		return -EINVAL;
+
 	pw_proxy_notify(proxy, struct pw_client_node_events, command, 0, command);
 	return 0;
 }
@@ -1117,6 +1103,9 @@ static int client_node_demarshal_event_method(void *object, const struct pw_prot
 			SPA_POD_PodObject(&event)) < 0)
 		return -EINVAL;
 
+	if (event == NULL)
+		return -EINVAL;
+
 	pw_resource_notify(resource, struct pw_client_node_methods, event, 0, event);
 	return 0;
 }
