@@ -12,14 +12,95 @@
 #include <spa/utils/result.h>
 #include <spa/utils/json.h>
 
+#define PW_API_METADATA_IMPL	SPA_EXPORT
 #include <pipewire/impl.h>
 #include <pipewire/extensions/metadata.h>
 
 /** \page page_module_metadata Metadata
+ *
+ * Allows clients to export a metadata store to the PipeWire server.
+ *
+ * Both the client and the server need to load this module for the metadata
+ * to be exported.
+ *
+ * This module creates a new factory and a new export type for the
+ * \ref PW_TYPE_INTERFACE_Metadata interface.
+ *
+ * A client will first create an implementation of the PW_TYPE_INTERFACE_Metadata
+ * interface with \ref pw_context_create_metadata(), for example. With the
+ * \ref pw_core_export(), this module will create a server side resource to expose
+ * the metadata implementation to other clients. Modifications done by the client
+ * on the local metadata interface will be visible to all PipeWire clients.
+ *
+ * It is also possible to use the factory to create metadata in the current
+ * processes using a config file fragment.
+ *
+ * As an argument to the create_object call, a set of properties will
+ * control the name of the metadata and some initial values.
  *
  * ## Module Name
  *
  * `libpipewire-module-metadata`
+ *
+ * ## Module Options
+ *
+ * This module has no options.
+ *
+ * ## Properties for the create_object call
+ *
+ * - `metadata.name`: The name of the new metadata object. If not given, the metadata
+ *                    object name will be `default`.
+ * - `metadata.values`: A JSON array of objects with initial values for the metadata object.
+ *
+ *   the `metadata.values` key has the following layout:
+ *
+ *  \code{.unparsed}
+ *   metadata.values = [
+ *      { id = <int>  key = <key>  type = <type> value = <object> }
+ *      ....
+ *   ]
+ *  \endcode
+ *
+ *     - `id`: an optional object id for the metadata, default 0
+ *     - `key`: a string, the metadata key
+ *     - `type`: an optional metadata value type
+ *     - `value`: a JSON item, the metadata value.
+ *
+ * ## Example configuration
+ *
+ * The module is usually added to the config file of the main PipeWire daemon and the
+ * clients.
+ *
+ *\code{.unparsed}
+ * context.modules = [
+ * { name = libpipewire-module-metadata }
+ * ]
+ *\endcode
+
+ * ## Config objects
+ *
+ * To create an object from the factory, one can use the \ref pw_core_create_object()
+ * method or make an object in the `context.objects` section like in the main PipeWire
+ * daemon config file:
+ *
+ *\code{.unparsed}
+ * context.objects = [
+ * { factory = metadata
+ *     args = {
+ *         metadata.name = default
+ *         metadata.values = [
+ *            { key = default.audio.sink   value = { name = somesink } }
+ *            { key = default.audio.source value = { name = somesource } }
+ *         ]
+ *     }
+ * }
+ *\endcode
+ *
+ * This creates a new default metadata store with 2 key/values.
+ *
+ * ## See also
+ *
+ * - `pw-metadata`: a tool to manage metadata
  */
 
 #define NAME "metadata"
@@ -66,23 +147,17 @@ struct factory_data {
  */
 static int fill_metadata(struct pw_metadata *metadata, const char *str)
 {
-	struct spa_json it[3];
+	struct spa_json it[2];
 
-	spa_json_init(&it[0], str, strlen(str));
-	if (spa_json_enter_array(&it[0], &it[1]) <= 0)
+	if (spa_json_begin_array(&it[0], str, strlen(str)) <= 0)
 		return -EINVAL;
 
-	while (spa_json_enter_object(&it[1], &it[2]) > 0) {
+	while (spa_json_enter_object(&it[0], &it[1]) > 0) {
 		char key[256], *k = NULL, *v = NULL, *t = NULL;
-		int id = 0;
-
-		while (spa_json_get_string(&it[2], key, sizeof(key)) > 0) {
-			int len;
-			const char *val;
-
-			if ((len = spa_json_next(&it[2], &val)) <= 0)
-				return -EINVAL;
+		int id = 0, len;
+		const char *val;
 
+		while ((len = spa_json_object_next(&it[1], key, sizeof(key), &val)) > 0) {
 			if (spa_streq(key, "id")) {
 				if (spa_json_parse_int(val, len, &id) <= 0)
 					return -EINVAL;
@@ -94,7 +169,7 @@ static int fill_metadata(struct pw_metadata *metadata, const char *str)
 					spa_json_parse_stringn(val, len, t, len+1);
 			} else if (spa_streq(key, "value")) {
 				if (spa_json_is_container(val, len))
-					len = spa_json_container_len(&it[2], val, len);
+					len = spa_json_container_len(&it[1], val, len);
 				if ((v = malloc(len+1)) != NULL)
 					spa_json_parse_stringn(val, len, v, len+1);
 			}
