@@ -8,6 +8,36 @@
 #include "../defs.h"
 #include "../module.h"
 
+/** \page page_pulse_module_rtp_send RTP Sender
+ *
+ * ## Module Name
+ *
+ * `module-rtp-send`
+ *
+ * ## Module Options
+ *
+ * @pulse_module_options@
+ *
+ * ## See Also
+ *
+ * \ref page_module_rtp_sink "libpipewire-module-rtp-sink"
+ */
+
+static const char *const pulse_module_options =
+	"source=<name of the source> "
+	"format=<sample format> "
+	"channels=<number of channels> "
+	"rate=<sample rate> "
+	"destination_ip=<destination IP address> "
+	"source_ip=<source IP address> "
+	"port=<port number> "
+	"mtu=<maximum transfer unit> "
+	"loop=<loopback to local host?> "
+	"ttl=<ttl value> "
+	"inhibit_auto_suspend=<always|never|only_with_non_monitor_sources> "
+	"stream_name=<name of the stream> "
+	"enable_opus=<enable OPUS codec>";
+
 #define NAME "rtp-send"
 
 PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
@@ -25,8 +55,6 @@ struct module_rtp_send_data {
 	struct pw_properties *stream_props;
 	struct pw_properties *global_props;
 	struct pw_properties *sap_props;
-
-	struct spa_audio_info_raw info;
 };
 
 static void module_destroy(void *data)
@@ -61,7 +89,6 @@ static int module_rtp_send_load(struct module *module)
 	FILE *f;
 	char *args;
 	size_t size;
-	uint32_t i;
 
 	pw_properties_setf(data->stream_props, "pulse.module.id",
 			"%u", module->index);
@@ -71,20 +98,6 @@ static int module_rtp_send_load(struct module *module)
 
 	fprintf(f, "{");
 	pw_properties_serialize_dict(f, &data->global_props->dict, 0);
-	if (data->info.format != 0)
-		fprintf(f, " \"audio.format\": \"%s\"", format_id2name(data->info.format));
-	if (data->info.rate != 0)
-		fprintf(f, " \"audio.rate\": %u,", data->info.rate);
-	if (data->info.channels != 0) {
-		fprintf(f, " \"audio.channels\": %u,", data->info.channels);
-		if (!(data->info.flags & SPA_AUDIO_FLAG_UNPOSITIONED)) {
-			fprintf(f, " \"audio.position\": [ ");
-			for (i = 0; i < data->info.channels; i++)
-				fprintf(f, "%s\"%s\"", i == 0 ? "" : ",",
-					channel_id2name(data->info.position[i]));
-			fprintf(f, " ],");
-		}
-	}
 	fprintf(f, " stream.props = {");
 	pw_properties_serialize_dict(f, &data->stream_props->dict, 0);
 	fprintf(f, " } }");
@@ -153,19 +166,7 @@ static int module_rtp_send_unload(struct module *module)
 static const struct spa_dict_item module_rtp_send_info[] = {
 	{ PW_KEY_MODULE_AUTHOR, "Wim Taymans <wim.taymans@gmail.com>" },
 	{ PW_KEY_MODULE_DESCRIPTION, "Read data from source and send it to the network via RTP/SAP/SDP" },
-	{ PW_KEY_MODULE_USAGE,	"source=<name of the source> "
-				"format=<sample format> "
-				"channels=<number of channels> "
-				"rate=<sample rate> "
-				"destination_ip=<destination IP address> "
-				"source_ip=<source IP address> "
-				"port=<port number> "
-				"mtu=<maximum transfer unit> "
-				"loop=<loopback to local host?> "
-				"ttl=<ttl value> "
-				"inhibit_auto_suspend=<always|never|only_with_non_monitor_sources> "
-				"stream_name=<name of the stream> "
-				"enable_opus=<enable OPUS codec>" },
+	{ PW_KEY_MODULE_USAGE, pulse_module_options },
 	{ PW_KEY_MODULE_VERSION, PACKAGE_VERSION },
 };
 
@@ -198,19 +199,13 @@ static int module_rtp_send_prepare(struct module * const module)
 			pw_properties_set(stream_props, PW_KEY_TARGET_OBJECT, str);
 		}
 	}
-	if (module_args_to_audioinfo(module->impl, props, &info) < 0) {
+
+	if (module_args_to_audioinfo_keys(module->impl, props,
+			"format", "rate", "channels", "channel_map", &info) < 0) {
 		res = -EINVAL;
 		goto out;
 	}
-	info.format = 0;
-	if ((str = pw_properties_get(props, "format")) != NULL) {
-		if ((info.format = format_paname2id(str, strlen(str))) ==
-				SPA_AUDIO_FORMAT_UNKNOWN) {
-			pw_log_error("unknown format %s", str);
-			res = -EINVAL;
-			goto out;
-		}
-	}
+	audioinfo_to_properties(&info, global_props);
 
 	pw_properties_set(global_props, "sess.media", "audio");
 	if ((str = pw_properties_get(props, "enable_opus")) != NULL) {
@@ -245,7 +240,6 @@ static int module_rtp_send_prepare(struct module * const module)
 	d->stream_props = stream_props;
 	d->global_props = global_props;
 	d->sap_props = sap_props;
-	d->info = info;
 
 	return 0;
 out:
