@@ -19,6 +19,12 @@ struct rtp_stream;
 #define ERROR_MSEC		2.0f
 #define DEFAULT_SESS_LATENCY	100.0f
 
+#define IP4_HEADER_SIZE		20
+#define IP6_HEADER_SIZE		40
+#define UDP_HEADER_SIZE		8
+/* 12 bytes RTP header */
+#define RTP_HEADER_SIZE		12
+
 #define DEFAULT_MTU		1280
 #define DEFAULT_MIN_PTIME	2.0f
 #define DEFAULT_MAX_PTIME	20.0f
@@ -52,8 +58,11 @@ uint64_t rtp_stream_get_time(struct rtp_stream *s, uint32_t *rate);
 
 uint16_t rtp_stream_get_seq(struct rtp_stream *s);
 
+size_t rtp_stream_get_mtu(struct rtp_stream *s);
+
 void rtp_stream_set_first(struct rtp_stream *s);
 
+int rtp_stream_set_active(struct rtp_stream *s, bool active);
 void rtp_stream_set_error(struct rtp_stream *s, int res, const char *error);
 enum pw_stream_state rtp_stream_get_state(struct rtp_stream *s, const char **error);
 
