@@ -8,7 +8,11 @@
 #include <spa/utils/ansi.h>
 #include <spa/utils/json.h>
 #include <spa/utils/string.h>
+#include <spa/utils/cleanup.h>
+#include <spa/utils/result.h>
+#include <spa/debug/log.h>
 
+#define PW_API_PROPERTIES SPA_EXPORT
 #include "pipewire/array.h"
 #include "pipewire/log.h"
 #include "pipewire/utils.h"
@@ -25,24 +29,38 @@ struct properties {
 };
 /** \endcond */
 
-static int add_func(struct pw_properties *this, char *key, char *value)
+static int add_item(struct properties *impl, const char *key, bool take_key, const char *value, bool take_value)
 {
 	struct spa_dict_item *item;
-	struct properties *impl = SPA_CONTAINER_OF(this, struct properties, this);
+	const char *k, *v;
+
+	k = take_key ? key : NULL;
+	v = take_value ? value: NULL;
+
+	if (!take_key && key && (k = strdup(key)) == NULL)
+		goto error;
+	if (!take_value && value && (v = strdup(value)) == NULL)
+		goto error;
 
 	item = pw_array_add(&impl->items, sizeof(struct spa_dict_item));
-	if (item == NULL) {
-		free(key);
-		free(value);
-		return -errno;
-	}
+	if (item == NULL)
+		goto error;
 
-	item->key = key;
-	item->value = value;
-
-	this->dict.items = impl->items.data;
-	this->dict.n_items++;
+	item->key = k;
+	item->value = v;
 	return 0;
+
+error:
+	free((char*)k);
+	free((char*)v);
+	return -errno;
+}
+
+static void update_dict(struct pw_properties *properties)
+{
+	struct properties *impl = SPA_CONTAINER_OF(properties, struct properties, this);
+	properties->dict.items = impl->items.data;
+	properties->dict.n_items = pw_array_get_len(&impl->items, struct spa_dict_item);
 }
 
 static void clear_item(struct spa_dict_item *item)
@@ -51,26 +69,20 @@ static void clear_item(struct spa_dict_item *item)
 	free((char *) item->value);
 }
 
-static int find_index(const struct pw_properties *this, const char *key)
+static void properties_init(struct properties *impl, int prealloc)
 {
-	const struct spa_dict_item *item;
-	item = spa_dict_lookup_item(&this->dict, key);
-	if (item == NULL)
-		return -1;
-	return item - this->dict.items;
+	pw_array_init(&impl->items, 16);
+	pw_array_ensure_size(&impl->items, sizeof(struct spa_dict_item) * prealloc);
 }
 
 static struct properties *properties_new(int prealloc)
 {
 	struct properties *impl;
 
-	impl = calloc(1, sizeof(struct properties));
-	if (impl == NULL)
+	if ((impl = calloc(1, sizeof(struct properties))) == NULL)
 		return NULL;
 
-	pw_array_init(&impl->items, 16);
-	pw_array_ensure_size(&impl->items, sizeof(struct spa_dict_item) * prealloc);
-
+	properties_init(impl, prealloc);
 	return impl;
 }
 
@@ -86,6 +98,7 @@ struct pw_properties *pw_properties_new(const char *key, ...)
 	struct properties *impl;
 	va_list varargs;
 	const char *value;
+	int res = 0;
 
 	impl = properties_new(16);
 	if (impl == NULL)
@@ -95,12 +108,18 @@ struct pw_properties *pw_properties_new(const char *key, ...)
 	while (key != NULL) {
 		value = va_arg(varargs, char *);
 		if (value && key[0])
-			add_func(&impl->this, strdup(key), strdup(value));
+			if ((res = add_item(impl, key, false, value, false)) < 0)
+				goto error;
 		key = va_arg(varargs, char *);
 	}
 	va_end(varargs);
+	update_dict(&impl->this);
 
 	return &impl->this;
+error:
+	pw_properties_free(&impl->this);
+	errno = -res;
+	return NULL;
 }
 
 /** Make a new properties object from the given dictionary
@@ -113,6 +132,7 @@ struct pw_properties *pw_properties_new_dict(const struct spa_dict *dict)
 {
 	uint32_t i;
 	struct properties *impl;
+	int res;
 
 	impl = properties_new(SPA_ROUND_UP_N(dict->n_items, 16));
 	if (impl == NULL)
@@ -121,11 +141,150 @@ struct pw_properties *pw_properties_new_dict(const struct spa_dict *dict)
 	for (i = 0; i < dict->n_items; i++) {
 		const struct spa_dict_item *it = &dict->items[i];
 		if (it->key != NULL && it->key[0] && it->value != NULL)
-			add_func(&impl->this, strdup(it->key),
-				 strdup(it->value));
+			if ((res = add_item(impl, it->key, false, it->value, false)) < 0)
+				goto error;
 	}
+	update_dict(&impl->this);
 
 	return &impl->this;
+
+error:
+	pw_properties_free(&impl->this);
+	errno = -res;
+	return NULL;
+}
+
+static int do_replace(struct pw_properties *properties, const char *key, bool take_key,
+		const char *value, bool take_value)
+{
+	struct properties *impl = SPA_CONTAINER_OF(properties, struct properties, this);
+	struct spa_dict_item *item;
+	int res = 0;
+
+	if (key == NULL || key[0] == 0)
+		goto exit_noupdate;
+
+	item = (struct spa_dict_item*) spa_dict_lookup_item(&properties->dict, key);
+
+	if (item == NULL) {
+		if (value == NULL)
+			goto exit_noupdate;
+		if ((res = add_item(impl, key, take_key, value, take_value)) < 0)
+			return res;
+		SPA_FLAG_CLEAR(properties->dict.flags, SPA_DICT_FLAG_SORTED);
+	} else {
+		if (value && spa_streq(item->value, value))
+			goto exit_noupdate;
+
+		if (value == NULL) {
+			struct spa_dict_item *last = pw_array_get_unchecked(&impl->items,
+						     pw_array_get_len(&impl->items, struct spa_dict_item) - 1,
+						     struct spa_dict_item);
+			clear_item(item);
+			item->key = last->key;
+			item->value = last->value;
+			impl->items.size -= sizeof(struct spa_dict_item);
+			SPA_FLAG_CLEAR(properties->dict.flags, SPA_DICT_FLAG_SORTED);
+		} else {
+			char *v = NULL;
+			if (!take_value && value && (v = strdup(value)) == NULL) {
+				res = -errno;
+				goto exit_noupdate;
+			}
+			free((char *) item->value);
+			item->value = take_value ? value : v;
+		}
+		if (take_key)
+			free((char*)key);
+	}
+	update_dict(properties);
+	return 1;
+
+exit_noupdate:
+	if (take_key)
+		free((char*)key);
+	if (take_value)
+		free((char*)value);
+	return res;
+}
+
+static int update_string(struct pw_properties *props, const char *str, size_t size,
+		int *count, struct spa_error_location *loc)
+{
+	struct spa_json it[1];
+	char key[1024];
+	struct spa_error_location el;
+	bool err;
+	int len, res, cnt = 0;
+	struct properties changes;
+	const char *value;
+
+	if ((res = spa_json_begin_object_relax(&it[0], str, size)) <= 0)
+		return res;
+
+	if (props)
+		properties_init(&changes, 16);
+
+	while ((len = spa_json_object_next(&it[0], key, sizeof(key), &value)) > 0) {
+		char *val = NULL;
+
+		if (spa_json_is_null(value, len))
+			val = NULL;
+		else {
+			if (spa_json_is_container(value, len))
+				len = spa_json_container_len(&it[0], value, len);
+			if (len <= 0)
+				break;
+
+			if (props) {
+				if ((val = malloc(len+1)) == NULL) {
+					it[0].state = SPA_JSON_ERROR_FLAG;
+					break;
+				}
+				spa_json_parse_stringn(value, len, val, len+1);
+			}
+		}
+		if (props) {
+			const struct spa_dict_item *item;
+			item = spa_dict_lookup_item(&props->dict, key);
+			if (item && spa_streq(item->value, val)) {
+				free(val);
+				continue;
+			}
+			/* item changed or added, apply changes later */
+			if ((errno = -add_item(&changes, key, false, val, true) < 0)) {
+				it[0].state = SPA_JSON_ERROR_FLAG;
+				break;
+			}
+		}
+	}
+	if ((err = spa_json_get_error(&it[0], str, &el))) {
+		if (loc == NULL)
+			spa_debug_log_error_location(pw_log_get(), SPA_LOG_LEVEL_WARN,
+					&el, "error parsing more than %d properties: %s",
+					cnt, el.reason);
+		else
+			*loc = el;
+	}
+	if (props) {
+		struct spa_dict_item *item;
+		if (loc == NULL || !err) {
+			pw_array_for_each(item, &changes.items)
+				if ((res = do_replace(props, item->key, true, item->value, true)) < 0)
+					pw_log_warn("error updating property: %s", spa_strerror(res));
+				else
+					cnt += res;
+
+		} else {
+			pw_array_for_each(item, &changes.items)
+				clear_item(item);
+		}
+		pw_array_clear(&changes.items);
+	}
+	if (count)
+		*count = cnt;
+
+	return !err;
 }
 
 /** Update the properties from the given string, overwriting any
@@ -139,34 +298,36 @@ struct pw_properties *pw_properties_new_dict(const struct spa_dict *dict)
 SPA_EXPORT
 int pw_properties_update_string(struct pw_properties *props, const char *str, size_t size)
 {
-	struct properties *impl = SPA_CONTAINER_OF(props, struct properties, this);
-	struct spa_json it[2];
-	char key[1024], *val;
 	int count = 0;
+	update_string(props, str, size, &count, NULL);
+	return count;
+}
 
-	spa_json_init(&it[0], str, size);
-	if (spa_json_enter_object(&it[0], &it[1]) <= 0)
-		spa_json_init(&it[1], str, size);
-
-	while (spa_json_get_string(&it[1], key, sizeof(key)) > 0) {
-		int len;
-		const char *value;
-
-		if ((len = spa_json_next(&it[1], &value)) <= 0)
-			break;
-
-		if (spa_json_is_null(value, len))
-			val = NULL;
-		else {
-			if (spa_json_is_container(value, len))
-				len = spa_json_container_len(&it[1], value, len);
-
-			if ((val = malloc(len+1)) != NULL)
-				spa_json_parse_stringn(value, len, val, len+1);
-		}
-		count += pw_properties_set(&impl->this, key, val);
-		free(val);
-	}
+/** Check \a str is a well-formed properties JSON string and update
+ * the properties on success.
+ *
+ * \a str should be a whitespace separated list of key=value
+ * strings or a json object, see pw_properties_new_string().
+ *
+ * When the check fails, this function will not update \a props.
+ *
+ * \param props  The properties to attempt to update, maybe be NULL
+ *            to simply check the JSON string.
+ * \param str  The JSON object with new values
+ * \param size  The length of the JSON string.
+ * \param loc  Return value for parse error location
+ * \return a negative value when string is not valid and \a loc contains
+ *         the error location or the number of updated properties in
+ *         \a props.
+ * \since 1.1.0
+ */
+SPA_EXPORT
+int pw_properties_update_string_checked(struct pw_properties *props,
+		const char *str, size_t size, struct spa_error_location *loc)
+{
+	int count = 0;
+	if (!update_string(props, str, size, &count, loc))
+		return -EINVAL;
 	return count;
 }
 
@@ -199,6 +360,27 @@ error:
 	return NULL;
 }
 
+SPA_EXPORT
+struct pw_properties *
+pw_properties_new_string_checked(const char *object, size_t size, struct spa_error_location *loc)
+{
+	struct properties *impl;
+	int res;
+
+	impl = properties_new(16);
+	if (impl == NULL)
+		return NULL;
+
+	if ((res = pw_properties_update_string_checked(&impl->this, object, size, loc)) < 0)
+		goto error;
+
+	return &impl->this;
+error:
+	pw_properties_free(&impl->this);
+	errno = -res;
+	return NULL;
+}
+
 /** Copy a properties object
  *
  * \param properties properties to copy
@@ -221,12 +403,17 @@ SPA_EXPORT
 int pw_properties_update_keys(struct pw_properties *props,
 		const struct spa_dict *dict, const char * const keys[])
 {
-	int i, changed = 0;
+	int i, res, changed = 0;
 	const char *str;
 
 	for (i = 0; keys[i]; i++) {
-		if ((str = spa_dict_lookup(dict, keys[i])) != NULL)
-			changed += pw_properties_set(props, keys[i], str);
+		if ((str = spa_dict_lookup(dict, keys[i])) != NULL) {
+			if ((res = pw_properties_set(props, keys[i], str)) < 0)
+				pw_log_warn("error updating property %s:%s: %s",
+						keys[i], str, spa_strerror(res));
+			else
+				changed += res;
+		}
 	}
 	return changed;
 }
@@ -246,11 +433,16 @@ int pw_properties_update_ignore(struct pw_properties *props,
 		const struct spa_dict *dict, const char * const ignore[])
 {
 	const struct spa_dict_item *it;
-	int changed = 0;
+	int res, changed = 0;
 
 	spa_dict_for_each(it, dict) {
-		if (ignore == NULL || !has_key(ignore, it->key))
-			changed += pw_properties_set(props, it->key, it->value);
+		if (ignore == NULL || !has_key(ignore, it->key)) {
+			if ((res = pw_properties_set(props, it->key, it->value)) < 0)
+				pw_log_warn("error updating property %s:%s: %s",
+						it->key, it->value, spa_strerror(res));
+			else
+				changed += res;
+		}
 	}
 	return changed;
 }
@@ -285,11 +477,15 @@ int pw_properties_update(struct pw_properties *props,
 		         const struct spa_dict *dict)
 {
 	const struct spa_dict_item *it;
-	int changed = 0;
-
-	spa_dict_for_each(it, dict)
-		changed += pw_properties_set(props, it->key, it->value);
+	int res, changed = 0;
 
+	spa_dict_for_each(it, dict) {
+		if ((res = pw_properties_set(props, it->key, it->value)) < 0)
+			pw_log_warn("error updating property %s:%s: %s",
+					it->key, it->value, spa_strerror(res));
+		else
+			changed += res;
+	}
 	return changed;
 }
 
@@ -306,11 +502,17 @@ int pw_properties_add(struct pw_properties *props,
 		         const struct spa_dict *dict)
 {
 	uint32_t i;
-	int added = 0;
+	int res, added = 0;
 
 	for (i = 0; i < dict->n_items; i++) {
-		if (pw_properties_get(props, dict->items[i].key) == NULL)
-			added += pw_properties_set(props, dict->items[i].key, dict->items[i].value);
+		const struct spa_dict_item *it = &dict->items[i];
+		if (pw_properties_get(props, it->key) == NULL) {
+			if ((res = pw_properties_set(props, it->key, it->value)) < 0)
+				pw_log_warn("error updating property %s:%s: %s",
+						it->key, it->value, spa_strerror(res));
+			else
+				added += res;
+		}
 	}
 	return added;
 }
@@ -330,14 +532,19 @@ int pw_properties_add_keys(struct pw_properties *props,
 		const struct spa_dict *dict, const char * const keys[])
 {
 	uint32_t i;
-	int added = 0;
+	int res, added = 0;
 	const char *str;
 
 	for (i = 0; keys[i]; i++) {
 		if ((str = spa_dict_lookup(dict, keys[i])) == NULL)
 			continue;
-		if (pw_properties_get(props, keys[i]) == NULL)
-			added += pw_properties_set(props, keys[i], str);
+		if (pw_properties_get(props, keys[i]) == NULL) {
+			if ((res = pw_properties_set(props, keys[i], str)) < 0)
+				pw_log_warn("error updating property %s:%s: %s",
+						keys[i], str, spa_strerror(res));
+			else
+				added += res;
+		}
 	}
 	return added;
 }
@@ -360,50 +567,6 @@ void pw_properties_free(struct pw_properties *properties)
 	free(impl);
 }
 
-static int do_replace(struct pw_properties *properties, const char *key, char *value, bool copy)
-{
-	struct properties *impl = SPA_CONTAINER_OF(properties, struct properties, this);
-	int index;
-
-	if (key == NULL || key[0] == 0)
-		goto exit_noupdate;
-
-	index = find_index(properties, key);
-
-	if (index == -1) {
-		if (value == NULL)
-			return 0;
-		add_func(properties, strdup(key), copy ? strdup(value) : value);
-		SPA_FLAG_CLEAR(properties->dict.flags, SPA_DICT_FLAG_SORTED);
-	} else {
-		struct spa_dict_item *item =
-		    pw_array_get_unchecked(&impl->items, index, struct spa_dict_item);
-
-		if (value && spa_streq(item->value, value))
-			goto exit_noupdate;
-
-		if (value == NULL) {
-			struct spa_dict_item *last = pw_array_get_unchecked(&impl->items,
-						     pw_array_get_len(&impl->items, struct spa_dict_item) - 1,
-						     struct spa_dict_item);
-			clear_item(item);
-			item->key = last->key;
-			item->value = last->value;
-			impl->items.size -= sizeof(struct spa_dict_item);
-			properties->dict.n_items--;
-			SPA_FLAG_CLEAR(properties->dict.flags, SPA_DICT_FLAG_SORTED);
-		} else {
-			free((char *) item->value);
-			item->value = copy ? strdup(value) : value;
-		}
-	}
-	return 1;
-exit_noupdate:
-	if (!copy)
-		free(value);
-	return 0;
-}
-
 /** Set a property value
  *
  * \param properties the properties to change
@@ -411,7 +574,7 @@ exit_noupdate:
  * \param value a value or NULL to remove the key
  * \return 1 if the properties were changed. 0 if nothing was changed because
  *  the property already existed with the same value or because the key to remove
- *  did not exist.
+ *  did not exist. < 0 if an error occurred and nothing was changed.
  *
  * Set the property in \a properties with \a key to \a value. Any previous value
  * of \a key will be overwritten. When \a value is NULL, the key will be
@@ -420,7 +583,7 @@ exit_noupdate:
 SPA_EXPORT
 int pw_properties_set(struct pw_properties *properties, const char *key, const char *value)
 {
-	return do_replace(properties, key, (char*)value, true);
+	return do_replace(properties, key, false, (char*)value, false);
 }
 
 SPA_EXPORT
@@ -432,7 +595,7 @@ int pw_properties_setva(struct pw_properties *properties,
 		if (vasprintf(&value, format, args) < 0)
 			return -errno;
 	}
-	return do_replace(properties, key, value, false);
+	return do_replace(properties, key, false, value, true);
 }
 
 /** Set a property value by format
@@ -472,13 +635,7 @@ int pw_properties_setf(struct pw_properties *properties, const char *key, const
 SPA_EXPORT
 const char *pw_properties_get(const struct pw_properties *properties, const char *key)
 {
-	struct properties *impl = SPA_CONTAINER_OF(properties, struct properties, this);
-	int index = find_index(properties, key);
-
-	if (index == -1)
-		return NULL;
-
-	return pw_array_get_unchecked(&impl->items, index, struct spa_dict_item)->value;
+	return spa_dict_lookup(&properties->dict, key);
 }
 
 /** Fetch a property as uint32_t.
@@ -706,8 +863,11 @@ static int dump(struct dump_config *c, int indent, struct spa_json *it, const ch
 	if (value == NULL || len == 0) {
 		fprintf(file, "%snull%s", LITERAL(c), NORMAL(c));
 	} else if (spa_json_is_container(value, len) && !c->recurse) {
-		len = spa_json_container_len(it, value, len);
-		fprintf(file, "%s%.*s%s", CONTAINER(c), len, value, NORMAL(c));
+		spa_json_enter_container(it, &sub, value[0]);
+		if (spa_json_container_len(&sub, value, len) == len)
+			fprintf(file, "%s%.*s%s", CONTAINER(c), len, value, NORMAL(c));
+		else
+			encode_string(c, STRING(c), value, len, NORMAL(c));
 	} else if (spa_json_is_array(value, len)) {
 		fprintf(file, "[");
 		spa_json_enter(it, &sub);
@@ -724,14 +884,12 @@ static int dump(struct dump_config *c, int indent, struct spa_json *it, const ch
 		fprintf(file, "{");
 		spa_json_enter(it, &sub);
 		indent += c->indent;
-		while (spa_json_get_string(&sub, key, sizeof(key)) > 0) {
+		while ((len = spa_json_object_next(&sub, key, sizeof(key), &value)) > 0) {
 			fprintf(file, "%s%s%*s",
 					count++ > 0 ? "," : "",
 					c->sep, indent, "");
 			encode_string(c, KEY(c), key, strlen(key), NORMAL(c));
 			fprintf(file, ": ");
-			if ((len = spa_json_next(&sub, &value)) <= 0)
-				break;
 			dump(c, indent, &sub, value, len);
 		}
 		indent -= c->indent;
@@ -782,15 +940,11 @@ int pw_properties_serialize_dict(FILE *f, const struct spa_dict *dict, uint32_t
 			fprintf(f, "%s%s%s: ", KEY(c), key, NORMAL(c));
 		}
 		value = it->value;
-
 		len = value ? strlen(value) : 0;
 		spa_json_init(&sub, value, len);
-		if ((len = spa_json_next(&sub, &value)) < 0)
+		if (c->recurse && spa_json_next(&sub, &value) < 0)
 			break;
 
-		if (!spa_json_is_container(value, len))
-			len = value ? strlen(value) : 0;
-
 		dump(c, c->indent, &sub, value, len);
 		count++;
 	}
