@@ -70,15 +70,15 @@ static void do_quit(void *data, int signal_number)
 
 static void show_help(const char *name, bool error)
 {
-        fprintf(error ? stderr : stdout, "%s [options]\n"
-             "  -h, --help                            Show this help\n"
-             "      --version                         Show version\n"
-             "  -n, --name                            Name to reserve (Audio0, Midi0, Video0, ..)\n"
-             "  -a, --appname                         Application Name (default %s)\n"
-             "  -p, --priority                        Priority (default %d)\n"
-             "  -m, --monitor                         Monitor only, don't try to acquire\n"
-             "  -r, --release                         Request release when busy\n",
-	     name, DEFAULT_APPNAME, DEFAULT_PRIORITY);
+	fprintf(error ? stderr : stdout, "%s [options]\n"
+		"  -h, --help                            Show this help\n"
+		"      --version                         Show version\n"
+		"  -n, --name                            Name to reserve (Audio0, Midi0, Video0, ..)\n"
+		"  -a, --appname                         Application Name (default %s)\n"
+		"  -p, --priority                        Priority (default %d)\n"
+		"  -m, --monitor                         Monitor only, don't try to acquire\n"
+		"  -r, --release                         Request release when busy\n",
+		name, DEFAULT_APPNAME, DEFAULT_PRIORITY);
 }
 
 int main(int argc, char *argv[])
@@ -142,8 +142,8 @@ int main(int argc, char *argv[])
 			return -1;
 		}
 	}
-	if (opt_name == NULL) {
-		fprintf(stderr, "name must be given\n");
+	if (opt_name == NULL || !rd_device_valid_device_name(opt_name)) {
+		fprintf(stderr, "valid name must be given\n");
 		return -1;
 	}
 
@@ -190,6 +190,11 @@ int main(int argc, char *argv[])
 			opt_appname,
 			opt_priority,
 			&reserve_callbacks, &impl);
+	if (!impl.device) {
+		res = -errno;
+		fprintf(stderr, "dbus connection failed: %m\n");
+		goto exit;
+	}
 
 	if (!opt_monitor) {
 		res = rd_device_acquire(impl.device);
